// TableHeaderIcons.java

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.table.*;

class TableHeaderIcons extends JFrame
{
   TableHeaderIcons (String title)
   {
      // Pass the title argument to the JFrame superclass so that the
      // contents of title appear in the JFrame's title bar.

      super (title);

      // Exit the application by calling System.exit when the user 
      // either selects Close from the System menu or clicks the X 
      // button.

      setDefaultCloseOperation (EXIT_ON_CLOSE);

      // Define the data model for the JTable.

      TableModel dataModel = new AbstractTableModel ()
      {
         final static int NROWS = 20;
         final static int NCOLS = 5;

         Object [][] rowData = new Object [NROWS] [];

         // This instance initializer creates the data when an object 
         // is created from this subclass of the AbstractTableModel 
         // class.

         {
            for (int i = 0; i < NROWS; i++)
                 rowData [i] = new Object [NCOLS];

            int x = 0;
            for (int row = 0; row < NROWS; row++)
                 for (int col = 0; col < NCOLS; col++)
                      rowData [row] [col] = new Integer (2 * x++);
         }

         public int getColumnCount ()
         {
            return NCOLS;
         }

         public int getRowCount ()
         {
            return NROWS;
         }

         public Object getValueAt (int row, int col)
         {
            return rowData [row] [col];
         }

         public boolean isCellEditable (int row, int col)
         {
            return false;
         }

         public void setValueAt (Object value, int row, int col)
         {
            rowData [row] [col] = value;
            fireTableCellUpdated (row, col);
         }
      };

      // Create the JTable and assign it the predefined data model.

      JTable table = new JTable (dataModel);

      // Assign a custom renderer to column A, so that an icon can
      // be displayed in column A's header.

      table.getColumn ("A").setHeaderRenderer (new MyRenderer ());

      // Create a JScrollPane and add the JTable to this container.

      JScrollPane sp = new JScrollPane (table);

      // Set the JTable's preferred "viewport" size for viewing
      // JTable data.

      table.setPreferredScrollableViewportSize (new Dimension (250, 
                                                               180));

      // Add the JScrollPane to the JFrame's content pane.

      getContentPane ().add (sp);

      // Set the size of the JFrame window.

      setSize (300, 250);

      // Show the JFrame window.

      setVisible (true);
   }

   public static void main (String [] args)
   {
      new TableHeaderIcons ("Table Header Icons");
   }
}

class MyRenderer implements TableCellRenderer
{
   // Create a JLabel for use as a renderer and pre-load this label
   // with an icon image.

   private JLabel l = new JLabel (new ImageIcon ("icon.gif"));

   public Component getTableCellRendererComponent (JTable table,
                                                   Object value,
                                                   boolean isSelected,
                                                   boolean hasFocus,
                                                   int row,
                                                   int column)
   {
      // Extract the original header renderer for this column.

      TableCellRenderer tcr = table.getTableHeader ()
                                   .getDefaultRenderer ();

      // Extract the component used to render the column header.

      Component c = tcr.getTableCellRendererComponent (table, value,
                                                       isSelected,
                                                       hasFocus,
                                                       row, column);

      // Establish the font, foreground color, and border for the 
      // JLabel so that the rendered header will look the same as the 
      // other rendered headers.

      l.setFont (c.getFont ());
      l.setForeground (c.getForeground ());
      l.setBorder (((JComponent) c).getBorder ());

      // Establish the column name.

      l.setText ((String) value);

      // Return the cached JLabel a the renderer for this column 
      // header.

      return l;
   }
}
